"""
Author: Michel Peltriaux
Organization: Struktur- und Genehmigungsdirektion Nord, Rhineland-Palatinate, Germany
Contact: michel.peltriaux@sgdnord.rlp.de
Created on: 09.08.21

"""
from django.contrib import messages
from django.db.models import Sum
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.contrib.auth.decorators import login_required
from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpRequest, Http404, JsonResponse
from django.shortcuts import render, get_object_or_404, redirect

from compensation.forms.forms import NewEcoAccountForm, EditEcoAccountForm
from compensation.forms.modalForms import NewStateModalForm, NewActionModalForm, NewDeadlineModalForm, \
    NewEcoAccountDocumentModalForm, RemoveCompensationActionModalForm, RemoveCompensationStateModalForm, \
    EditCompensationStateModalForm, EditCompensationActionModalForm, EditDeadlineModalForm
from compensation.models import EcoAccount, EcoAccountDocument, CompensationState, CompensationAction
from compensation.tables import EcoAccountTable
from intervention.forms.modalForms import NewDeductionModalForm, ShareModalForm, RemoveEcoAccountDeductionModalForm, \
    EditEcoAccountDeductionModalForm
from konova.contexts import BaseContext
from konova.decorators import any_group_check, default_group_required, conservation_office_group_required, \
    shared_access_required
from konova.forms import RemoveModalForm, SimpleGeomForm, NewDocumentModalForm, RecordModalForm, \
    RemoveDeadlineModalForm, EditDocumentModalForm, ResubmissionModalForm
from konova.models import Deadline
from konova.settings import DEFAULT_GROUP, ZB_GROUP, ETS_GROUP
from konova.sub_settings.context_settings import TAB_TITLE_IDENTIFIER
from konova.utils.documents import get_document, remove_document
from konova.utils.generators import generate_qr_code
from konova.utils.message_templates import IDENTIFIER_REPLACED, FORM_INVALID, DATA_UNSHARED, DATA_UNSHARED_EXPLANATION, \
    CANCEL_ACC_RECORDED_OR_DEDUCTED, DEDUCTION_REMOVED, DEDUCTION_ADDED, DOCUMENT_ADDED, COMPENSATION_STATE_REMOVED, \
    COMPENSATION_STATE_ADDED, COMPENSATION_ACTION_REMOVED, COMPENSATION_ACTION_ADDED, DEADLINE_ADDED, DEADLINE_REMOVED, \
    DEDUCTION_EDITED, DOCUMENT_EDITED, COMPENSATION_STATE_EDITED, COMPENSATION_ACTION_EDITED, DEADLINE_EDITED, \
    RECORDED_BLOCKS_EDIT
from konova.utils.user_checks import in_group


@login_required
@any_group_check
def index_view(request: HttpRequest):
    """
    Renders the index view for eco accounts

    Args:
        request (HttpRequest): The incoming request

    Returns:
        A rendered view
    """
    template = "generic_index.html"
    eco_accounts = EcoAccount.objects.filter(
        deleted=None,
    )
    table = EcoAccountTable(
        request=request,
        queryset=eco_accounts
    )
    context = {
        "table": table,
        TAB_TITLE_IDENTIFIER: _("Eco-account - Overview"),
    }
    context = BaseContext(request, context).context
    return render(request, template, context)


@login_required
@default_group_required
def new_view(request: HttpRequest):
    """
    Renders a view for a new eco account creation

    Args:
        request (HttpRequest): The incoming request

    Returns:

    """
    template = "compensation/form/view.html"
    data_form = NewEcoAccountForm(request.POST or None)
    geom_form = SimpleGeomForm(request.POST or None, read_only=False)
    if request.method == "POST":
        if data_form.is_valid() and geom_form.is_valid():
            generated_identifier = data_form.cleaned_data.get("identifier", None)
            acc = data_form.save(request.user, geom_form)
            if generated_identifier != acc.identifier:
                messages.info(
                    request,
                    IDENTIFIER_REPLACED.format(
                        generated_identifier,
                        acc.identifier
                    )
                )
            messages.success(request, _("Eco-Account {} added").format(acc.identifier))
            return redirect("compensation:acc:detail", id=acc.id)
        else:
            messages.error(request, FORM_INVALID, extra_tags="danger",)
    else:
        # For clarification: nothing in this case
        pass
    context = {
        "form": data_form,
        "geom_form": geom_form,
        TAB_TITLE_IDENTIFIER: _("New Eco-Account"),
    }
    context = BaseContext(request, context).context
    return render(request, template, context)


@login_required
@default_group_required
def new_id_view(request: HttpRequest):
    """ JSON endpoint

    Provides fetching of free identifiers for e.g. AJAX calls

    """
    tmp = EcoAccount()
    identifier = tmp.generate_new_identifier()
    while EcoAccount.objects.filter(identifier=identifier).exists():
        identifier = tmp.generate_new_identifier()
    return JsonResponse(
        data={
            "gen_data": identifier
        }
    )


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def edit_view(request: HttpRequest, id: str):
    """
    Renders a view for editing compensations

    Args:
        request (HttpRequest): The incoming request

    Returns:

    """
    template = "compensation/form/view.html"
    # Get object from db
    acc = get_object_or_404(EcoAccount, id=id)
    if acc.is_recorded:
        messages.info(
            request,
            RECORDED_BLOCKS_EDIT
        )
        return redirect("compensation:acc:detail", id=id)

    # Create forms, initialize with values from db/from POST request
    data_form = EditEcoAccountForm(request.POST or None, instance=acc)
    geom_form = SimpleGeomForm(request.POST or None, read_only=False, instance=acc)
    if request.method == "POST":
        if data_form.is_valid() and geom_form.is_valid():
            # The data form takes the geom form for processing, as well as the performing user
            acc = data_form.save(request.user, geom_form)
            messages.success(request, _("Eco-Account {} edited").format(acc.identifier))
            return redirect("compensation:acc:detail", id=acc.id)
        else:
            messages.error(request, FORM_INVALID, extra_tags="danger",)
    else:
        # For clarification: nothing in this case
        pass
    context = {
        "form": data_form,
        "geom_form": geom_form,
        TAB_TITLE_IDENTIFIER: _("Edit {}").format(acc.identifier),
    }
    context = BaseContext(request, context).context
    return render(request, template, context)


@login_required
@any_group_check
def detail_view(request: HttpRequest, id: str):
    """ Renders a detail view for a compensation

    Args:
        request (HttpRequest): The incoming request
        id (str): The compensation's id

    Returns:

    """
    template = "compensation/detail/eco_account/view.html"
    acc = get_object_or_404(
        EcoAccount.objects.prefetch_related(
            "deadlines",
        ).select_related(
            'geometry',
            'responsible',
        ),
        id=id
    )
    geom_form = SimpleGeomForm(instance=acc)
    parcels = acc.get_underlying_parcels()
    _user = request.user
    is_data_shared = acc.is_shared_with(_user)

    # Order states according to surface
    before_states = acc.before_states.order_by("-surface")
    after_states = acc.after_states.order_by("-surface")

    # Precalculate logical errors between before- and after-states
    # Sum() returns None in case of no states, so we catch that and replace it with 0 for easier handling
    sum_before_states = before_states.aggregate(Sum("surface"))["surface__sum"] or 0
    sum_after_states = after_states.aggregate(Sum("surface"))["surface__sum"] or 0
    diff_states = abs(sum_before_states - sum_after_states)
    # Calculate rest of available surface for deductions
    available_total, available_relative = acc.get_available_rest()

    # Prefetch related data to decrease the amount of db connections
    deductions = acc.deductions.filter(
        intervention__deleted=None,
    )
    actions = acc.actions.all()

    request = acc.set_status_messages(request)

    context = {
        "obj": acc,
        "geom_form": geom_form,
        "parcels": parcels,
        "has_access": is_data_shared,
        "before_states": before_states,
        "after_states": after_states,
        "sum_before_states": sum_before_states,
        "sum_after_states": sum_after_states,
        "diff_states": diff_states,
        "available": available_relative,
        "available_total": available_total,
        "is_default_member": in_group(_user, DEFAULT_GROUP),
        "is_zb_member": in_group(_user, ZB_GROUP),
        "is_ets_member": in_group(_user, ETS_GROUP),
        "LANIS_LINK": acc.get_LANIS_link(),
        "deductions": deductions,
        "actions": actions,
        TAB_TITLE_IDENTIFIER: f"{acc.identifier} - {acc.title}",
        "has_finished_deadlines": acc.get_finished_deadlines().exists(),
    }
    context = BaseContext(request, context).context
    return render(request, template, context)


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def remove_view(request: HttpRequest, id: str):
    """ Renders a modal view for removing the eco account

    Args:
        request (HttpRequest): The incoming request
        id (str): The account's id

    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)

    # If the eco account has already been recorded OR there are already deductions, it can not be deleted by a regular
    # default group user
    if acc.recorded is not None or acc.deductions.exists():
        user = request.user
        if not in_group(user, ETS_GROUP):
            messages.info(request, CANCEL_ACC_RECORDED_OR_DEDUCTED)
            return redirect("compensation:acc:detail", id=id)

    form = RemoveModalForm(request.POST or None, instance=acc, request=request)
    return form.process_request(
        request=request,
        msg_success=_("Eco-account removed"),
        redirect_url=reverse("compensation:acc:index"),
    )


@login_required
@default_group_required
def deduction_remove_view(request: HttpRequest, id: str, deduction_id: str):
    """ Renders a modal view for removing deductions

    Args:
        request (HttpRequest): The incoming request
        id (str): The eco account's id
        deduction_id (str): The deduction's id

    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)
    try:
        eco_deduction = acc.deductions.get(id=deduction_id)
        if not eco_deduction.intervention.is_shared_with(request.user):
            raise ObjectDoesNotExist()
    except ObjectDoesNotExist:
        raise Http404("Unknown deduction")

    form = RemoveEcoAccountDeductionModalForm(request.POST or None, instance=acc, deduction=eco_deduction, request=request)
    return form.process_request(
        request=request,
        msg_success=DEDUCTION_REMOVED,
        redirect_url=reverse("compensation:acc:detail", args=(id,)) + "#related_data"
    )


@login_required
@default_group_required
def deduction_edit_view(request: HttpRequest, id: str, deduction_id: str):
    """ Renders a modal view for editing deductions

    Args:
        request (HttpRequest): The incoming request
        id (str): The eco account's id
        deduction_id (str): The deduction's id

    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)
    try:
        eco_deduction = acc.deductions.get(id=deduction_id)
        if not eco_deduction.intervention.is_shared_with(request.user):
            raise ObjectDoesNotExist
    except ObjectDoesNotExist:
        raise Http404("Unknown deduction")

    form = EditEcoAccountDeductionModalForm(request.POST or None, instance=acc, deduction=eco_deduction, request=request)
    return form.process_request(
        request=request,
        msg_success=DEDUCTION_EDITED,
        redirect_url=reverse("compensation:acc:detail", args=(id,)) + "#related_data"
    )


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def log_view(request: HttpRequest, id: str):
    """ Renders a log view using modal

    Args:
        request (HttpRequest): The incoming request
        id (str): The eco acount's id

    Returns:

    """
    comp = get_object_or_404(EcoAccount, id=id)
    template = "modal/modal_generic.html"
    body_template = "log.html"

    context = {
        "modal_body_template": body_template,
        "log": comp.log.all(),
        "modal_title": _("Log"),
    }
    context = BaseContext(request, context).context
    return render(request, template, context)


@login_required
@conservation_office_group_required
@shared_access_required(EcoAccount, "id")
def record_view(request: HttpRequest, id:str):
    """ Renders a modal form for recording an eco account

    Args:
        request (HttpRequest): The incoming request
        id (str): The account's id

    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)
    form = RecordModalForm(request.POST or None, instance=acc, request=request)
    msg_succ = _("{} unrecorded") if acc.recorded else _("{} recorded")
    msg_succ = msg_succ.format(acc.identifier)
    return form.process_request(
        request,
        msg_succ
    )


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def state_new_view(request: HttpRequest, id: str):
    """ Renders a form for adding new states for an eco account

    Args:
        request (HttpRequest): The incoming request
        id (str): The account's id to which the new state will be related

    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)
    form = NewStateModalForm(request.POST or None, instance=acc, request=request)
    return form.process_request(
        request,
        msg_success=COMPENSATION_STATE_ADDED,
        redirect_url=reverse("compensation:acc:detail", args=(id,)) + "#related_data"
    )


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def action_new_view(request: HttpRequest, id: str):
    """ Renders a form for adding new actions for an eco account

    Args:
        request (HttpRequest): The incoming request
        id (str): The account's id to which the new state will be related

    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)
    form = NewActionModalForm(request.POST or None, instance=acc, request=request)
    return form.process_request(
        request,
        msg_success=COMPENSATION_ACTION_ADDED,
        redirect_url=reverse("compensation:acc:detail", args=(id,)) + "#related_data"
    )


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def state_remove_view(request: HttpRequest, id: str, state_id: str):
    """ Renders a form for removing a compensation state

    Args:
        request (HttpRequest): The incoming request
        id (str): The compensation's id
        state_id (str): The state's id

    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)
    state = get_object_or_404(CompensationState, id=state_id)
    form = RemoveCompensationStateModalForm(request.POST or None, instance=acc, state=state, request=request)
    return form.process_request(
        request,
        msg_success=COMPENSATION_STATE_REMOVED,
        redirect_url=reverse("compensation:acc:detail", args=(id,)) + "#related_data"
    )


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def state_edit_view(request: HttpRequest, id: str, state_id: str):
    """ Renders a form for editing a compensation state

    Args:
        request (HttpRequest): The incoming request
        id (str): The compensation's id
        state_id (str): The state's id

    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)
    state = get_object_or_404(CompensationState, id=state_id)
    form = EditCompensationStateModalForm(request.POST or None, instance=acc, state=state, request=request)
    return form.process_request(
        request,
        msg_success=COMPENSATION_STATE_EDITED,
        redirect_url=reverse("compensation:acc:detail", args=(id,)) + "#related_data"
    )


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def action_remove_view(request: HttpRequest, id: str, action_id: str):
    """ Renders a form for removing a compensation action

    Args:
        request (HttpRequest): The incoming request
        id (str): The compensation's id
        id (str): The action's id

    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)
    action = get_object_or_404(CompensationAction, id=action_id)
    form = RemoveCompensationActionModalForm(request.POST or None, instance=acc, action=action, request=request)
    return form.process_request(
        request,
        msg_success=COMPENSATION_ACTION_REMOVED,
        redirect_url=reverse("compensation:acc:detail", args=(id,)) + "#related_data"
    )


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def action_edit_view(request: HttpRequest, id: str, action_id: str):
    """ Renders a form for editing a compensation action

    Args:
        request (HttpRequest): The incoming request
        id (str): The compensation's id
        id (str): The action's id

    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)
    action = get_object_or_404(CompensationAction, id=action_id)
    form = EditCompensationActionModalForm(request.POST or None, instance=acc, action=action, request=request)
    return form.process_request(
        request,
        msg_success=COMPENSATION_ACTION_EDITED,
        redirect_url=reverse("compensation:acc:detail", args=(id,)) + "#related_data"
    )


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def deadline_edit_view(request: HttpRequest, id: str, deadline_id: str):
    """ Renders a form for editing deadlines from a compensation

    Args:
        request (HttpRequest): The incoming request
        id (str): The compensation's id
        deadline_id (str): The deadline's id

    Returns:

    """
    comp = get_object_or_404(EcoAccount, id=id)
    deadline = get_object_or_404(Deadline, id=deadline_id)
    form = EditDeadlineModalForm(request.POST or None, instance=comp, deadline=deadline, request=request)
    return form.process_request(
        request,
        msg_success=DEADLINE_EDITED,
        redirect_url=reverse("compensation:acc:detail", args=(id,)) + "#related_data"
    )


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def deadline_remove_view(request: HttpRequest, id: str, deadline_id: str):
    """ Renders a form for removing deadlines from a compensation

    Args:
        request (HttpRequest): The incoming request
        id (str): The compensation's id
        deadline_id (str): The deadline's id

    Returns:

    """
    comp = get_object_or_404(EcoAccount, id=id)
    deadline = get_object_or_404(Deadline, id=deadline_id)
    form = RemoveDeadlineModalForm(request.POST or None, instance=comp, deadline=deadline, request=request)
    return form.process_request(
        request,
        msg_success=DEADLINE_REMOVED,
        redirect_url=reverse("compensation:acc:detail", args=(id,)) + "#related_data"
    )


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def deadline_new_view(request: HttpRequest, id: str):
    """ Renders a form for adding new states for an eco account

    Args:
        request (HttpRequest): The incoming request
        id (str): The account's id to which the new state will be related

    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)
    form = NewDeadlineModalForm(request.POST or None, instance=acc, request=request)
    return form.process_request(
        request,
        msg_success=DEADLINE_ADDED,
        redirect_url=reverse("compensation:acc:detail", args=(id,)) + "#related_data"
    )


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def new_document_view(request: HttpRequest, id: str):
    """ Renders a form for uploading new documents

    Args:
        request (HttpRequest): The incoming request
        id (str): The account's id to which the new document will be related
    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)
    form = NewEcoAccountDocumentModalForm(request.POST or None, request.FILES or None, instance=acc, request=request)
    return form.process_request(
        request,
        msg_success=DOCUMENT_ADDED,
        redirect_url=reverse("compensation:acc:detail", args=(id,)) + "#related_data",
    )


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def get_document_view(request: HttpRequest, id:str, doc_id: str):
    """ Returns the document as downloadable file

    Wraps the generic document fetcher function from konova.utils.

    Args:
        request (HttpRequest): The incoming request
        id (str): The account id
        doc_id (str): The document id

    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)
    doc = get_object_or_404(EcoAccountDocument, id=doc_id)
    return get_document(doc)


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def edit_document_view(request: HttpRequest, id: str, doc_id: str):
    """ Removes the document from the database and file system

    Wraps the generic functionality from konova.utils.

    Args:
        request (HttpRequest): The incoming request
        id (str): The account id
        doc_id (str): The document id

    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)
    doc = get_object_or_404(EcoAccountDocument, id=doc_id)
    form = EditDocumentModalForm(request.POST or None, request.FILES or None, instance=acc, document=doc, request=request)
    return form.process_request(
        request,
        DOCUMENT_EDITED,
        reverse("compensation:acc:detail", args=(id,)) + "#related_data"
    )


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def remove_document_view(request: HttpRequest, id: str, doc_id: str):
    """ Removes the document from the database and file system

    Wraps the generic functionality from konova.utils.

    Args:
        request (HttpRequest): The incoming request
        id (str): The account id
        doc_id (str): The document id

    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)
    doc = get_object_or_404(EcoAccountDocument, id=doc_id)
    return remove_document(
        request,
        doc
    )


@login_required
@default_group_required
def new_deduction_view(request: HttpRequest, id: str):
    """ Renders a modal form view for creating deductions

    Args:
        request (HttpRequest): THe incoming request
        id (str): The eco account's id

    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)
    if not acc.recorded:
        raise Http404()
    form = NewDeductionModalForm(request.POST or None, instance=acc, request=request)
    return form.process_request(
        request,
        msg_success=DEDUCTION_ADDED,
        redirect_url=reverse("compensation:acc:detail", args=(id,)) + "#related_data"
    )


def report_view(request:HttpRequest, id: str):
    """ Renders the public report view

    Args:
        request (HttpRequest): The incoming request
        id (str): The id of the intervention

    Returns:

    """
    # Reuse the compensation report template since EcoAccounts are structurally identical
    template = "compensation/report/eco_account/report.html"
    acc = get_object_or_404(EcoAccount, id=id)

    tab_title = _("Report {}").format(acc.identifier)
    # If intervention is not recorded (yet or currently) we need to render another template without any data
    if not acc.is_ready_for_publish():
        template = "report/unavailable.html"
        context = {
            TAB_TITLE_IDENTIFIER: tab_title,
        }
        context = BaseContext(request, context).context
        return render(request, template, context)

    # Prepare data for map viewer
    geom_form = SimpleGeomForm(
        instance=acc
    )
    parcels = acc.get_underlying_parcels()

    qrcode_url = request.build_absolute_uri(reverse("ema:report", args=(id,)))
    qrcode_img = generate_qr_code(qrcode_url, 10)
    qrcode_lanis_url = acc.get_LANIS_link()
    qrcode_img_lanis = generate_qr_code(qrcode_lanis_url, 7)

    # Order states by surface
    before_states = acc.before_states.all().order_by("-surface").select_related("biotope_type__parent")
    after_states = acc.after_states.all().order_by("-surface").select_related("biotope_type__parent")
    actions = acc.actions.all().prefetch_related("action_type__parent")

    # Reduce amount of db fetched data to the bare minimum we need in the template (deduction's intervention id and identifier)
    deductions = acc.deductions.all()\
        .distinct("intervention")\
        .select_related("intervention")\
        .values_list("intervention__id", "intervention__identifier", "intervention__title", named=True)

    context = {
        "obj": acc,
        "qrcode": {
            "img": qrcode_img,
            "url": qrcode_url,
        },
        "qrcode_lanis": {
            "img": qrcode_img_lanis,
            "url": qrcode_lanis_url,
        },
        "has_access": False,  # disables action buttons during rendering
        "before_states": before_states,
        "after_states": after_states,
        "geom_form": geom_form,
        "parcels": parcels,
        "actions": actions,
        "deductions": deductions,
        TAB_TITLE_IDENTIFIER: tab_title,
    }
    context = BaseContext(request, context).context
    return render(request, template, context)


@login_required
def share_view(request: HttpRequest, id: str, token: str):
    """ Performs sharing of an eco account

    If token given in url is not valid, the user will be redirected to the dashboard

    Args:
        request (HttpRequest): The incoming request
        id (str): EcoAccount's id
        token (str): Access token for EcoAccount

    Returns:

    """
    user = request.user
    obj = get_object_or_404(EcoAccount, id=id)
    # Check tokens
    if obj.access_token == token:
        # Send different messages in case user has already been added to list of sharing users
        if obj.is_shared_with(user):
            messages.info(
                request,
                _("{} has already been shared with you").format(obj.identifier)
            )
        else:
            messages.success(
                request,
                _("{} has been shared with you").format(obj.identifier)
            )
            obj.share_with_user(user)
        return redirect("compensation:acc:detail", id=id)
    else:
        messages.error(
            request,
            _("Share link invalid"),
            extra_tags="danger",
        )
        return redirect("home")


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def create_share_view(request: HttpRequest, id: str):
    """ Renders sharing form for an eco account

    Args:
        request (HttpRequest): The incoming request
        id (str): EcoAccount's id

    Returns:

    """
    obj = get_object_or_404(EcoAccount, id=id)
    form = ShareModalForm(request.POST or None, instance=obj, request=request)
    return form.process_request(
        request,
        msg_success=_("Share settings updated")
    )


@login_required
@default_group_required
@shared_access_required(EcoAccount, "id")
def create_resubmission_view(request: HttpRequest, id: str):
    """ Renders resubmission form for an eco account

    Args:
        request (HttpRequest): The incoming request
        id (str): EcoAccount's id

    Returns:

    """
    acc = get_object_or_404(EcoAccount, id=id)
    form = ResubmissionModalForm(request.POST or None, instance=acc, request=request)
    form.action_url = reverse("compensation:acc:resubmission-create", args=(id,))
    return form.process_request(
        request,
        msg_success=_("Resubmission set"),
        redirect_url=reverse("compensation:acc:detail", args=(id,))
    )